#!/usr/bin/python3

# This python module listens for publications from a MQTT broker,
# and enters the received document in the CMS document database
# Written by Anders Fongen, FHS/CIS, sept 2019

# TODO rewrite sql execution to protect against SQLinjection attacks

import paho.mqtt.client as mqtt
import json
import sqlite3
import sys
TOPIC="cms/sensors"
DBFILE="/var/www/html/cms/cmsdocs.db"
PSBROKER="localhost"
CLIENTNAME="Autogenerator"
INSERTMARK = "<!-- Insert -->"
class Document():

    def __init__(self,jsonTxt):
        self.docObj = json.loads(jsonTxt)

    def getTitle(self): return self.docObj['title']
    def getBody(self): return self.docObj['body']
    def getDocid(self): return int(self.docObj['id'])
    def getParent(self): return int(self.docObj['pid'])
    def getRefresh(self): return int(self.docObj['refresh'])

class SQLhandler():

    def __init__(self,dbfile):
        print ("Opening database")
        self.conn = sqlite3.connect(dbfile)

# we don't allow received documents to overwrite "human" documents,
# only automatic. By looking at the "protected" column, we can check
# if the document exists, and the value of this column.
    def findProtectionById(self,id):
        cur = self.conn.cursor()
        cur.execute("select protected,body from docs where id=%d" % id)
        row = cur.fetchone()
        cur.close()
        if row == None: return None
        else: return row[0],row[1]

# If the received document id already exists in the CMS document database
# and the protected column is >0, the received document will modifu that
# in the CMS database according to the following rules: (1) If the string
# <!-- Insert --> occurs in the existing document, the new content will be
# added at that point. (2) Otherwise, the new content will replace the
# the existing document. 
    def updateRow(self,docobj,existingDoc):
        print ("Update row")
        # Calculate new document content, based on rules set above
        if existingDoc.find(INSERTMARK) == -1:
#            print("Insertmark NOT found")
            newDoc = docobj.getBody()
        else:
            newDoc = existingDoc.replace(INSERTMARK,docobj.getBody())
#            print("Insertmark found")
#        print("Update to",newDoc)
        cur = self.conn.cursor()
        cur.execute("update docs set parent_doc=%d,title='%s',body='%s',refresh_rate=%d,last_edit=CURRENT_TIMESTAMP,protected=1,is_formatted=0 where id=%d" % \
           (docobj.getParent(), docobj.getTitle(), newDoc, docobj.getRefresh(),docobj.getDocid()))
        self.conn.commit()
        cur.close()

# If the received document id does not exist in the CMS database, the
# document will be added to the database.
    def insertRow(self,docobj):
        print ("Insert row")
        cur = self.conn.cursor()
        cur.execute("insert into docs (parent_doc,title,body,refresh_rate,protected ) values (%d,'%s','%s',%d,1)" % \
           (docobj.getParent(), docobj.getTitle(), docobj.getBody(), docobj.getRefresh()))
        self.conn.commit()
        cur.close()

    def closeConnection(self):
        print ("Lukker databasen")
        self.conn.close() 




# This class is the main thread in the program, which connects to the MQTT
# browser, receives documents and adds/updates them into the CMS database
class DocumentReceiver():
    def __init__(self,broker=PSBROKER,clientname=CLIENTNAME,dbfile=DBFILE):
        self.sqlhandler = SQLhandler(dbfile)
        self.client = mqtt.Client(clientname)
        self.client.connect(broker)
        self.client.subscribe(TOPIC)
        self.client.on_message=self.on_message
        print ("Ready to receive")
        self.client.loop_forever()

    def on_message(self,client,userdata,message):
        print ("Message received")
        try:
            doc = Document(str(message.payload.decode("utf-8")))
            # Enter data into database
            p,existingDoc = self.sqlhandler.findProtectionById(doc.getDocid())
            if p==None: self.sqlhandler.insertRow(doc)
            # Overwrite content of "automatic" document
            elif p != 0: self.sqlhandler.updateRow(doc,existingDoc)
            # Do not overwrite human generated documents
        except:
           e = sys.exc_info()[0]
           print("Error:",e)

if __name__ == "__main__":

    dr = DocumentReceiver()

    try :
       sqlhandler = SQLhandler(DBFILE)
       docstring = '{"title":"Autodok","body":"Her er ny xxxautotekst\'\\; lagt inn\\n\\n<!-- Insert -->","id":25,"pid":16,"refresh":0}'
       doc = Document(docstring)
       p,existingDoc = sqlhandler.findProtectionById(doc.getDocid())
       if p==None: sqlhandler.insertRow(doc)
       # Overwrite content of "automatic" document
       elif p != 0: sqlhandler.updateRow(doc,existingDoc)
       # Do not overwrite human generated documents
       sqlhandler.closeConnection()
    except:
       e = sys.exc_info()[0]
       print("Error:", e)



